/*LICENSE_START*/
/*
 *  Copyright 1995-2002 Washington University School of Medicine
 *
 *  http://brainmap.wustl.edu
 *
 *  This file is part of CARET.
 *
 *  CARET is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  CARET is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CARET; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/*LICENSE_END*/

#include "CommandStatisticalUnitTesting.h"
#include "ProgramParameters.h"
#include "ScriptBuilderParameters.h"
#include "StatisticUnitTesting.h"

/**
 * constructor.
 */
CommandStatisticalUnitTesting::CommandStatisticalUnitTesting()
   : CommandBase("-statistic-unit-test",
                 "STATISTICS UNIT TESTING")
{
}

/**
 * destructor.
 */
CommandStatisticalUnitTesting::~CommandStatisticalUnitTesting()
{
}

/**
 * get the script builder parameters.
 */
void 
CommandStatisticalUnitTesting::getScriptBuilderParameters(ScriptBuilderParameters& paramsOut) const
{
   paramsOut.clear();
   paramsOut.addBoolean("Show Values", false);
}
/**
 * get full help information.
 */
QString 
CommandStatisticalUnitTesting::getHelpInformation() const
{
   QString helpInfo =
      (indent3 + getShortDescription() + "\n"
       + indent6 + parameters->getProgramNameWithoutPath() + " " + getOperationSwitch() + "  \n"
       + indent9 + "<show-values-flag>\n"
       + indent9 + "\n"
       + indent9 + "Perform unit testing on statistical algorithm code.\n"
       + indent9 + "\n"
       + indent9 + "\"show-values-flag\" is either \"true\"or \"false\".  If true,\n"
       + indent9 + "all results are displayed, else only errors are displayed.\n"
       + indent9 + "\n");
      
   return helpInfo;
}

/**
 * execute the command.
 */
void 
CommandStatisticalUnitTesting::executeCommand() throw (BrainModelAlgorithmException,
                                     CommandException,
                                     FileException,
                                     ProgramParametersException,
                                     StatisticException)
{
   const bool showValuesFlag =
      parameters->getNextParameterAsBoolean("Show Values Flag");
      
   StatisticUnitTesting sut(showValuesFlag);
   sut.execute();
}


      

